import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.awt.Font;

/**
 * Like the carcass, the alphabet tile comes onto the screen, falls to the ground, and slowly
 * disappears.
 */
public class AlphabetTiles  extends Actor
{
    int square_size = 20;     //// Change for a bigger or smaller image.
    Font font = new Font("arial", Font.BOLD, 10);
    GreenfootImage image = new GreenfootImage(square_size, square_size);

    float ys;
    int y;

    int has_been_on_screen = 0;

    static final int allotted_time = 100; //Change to make the image stay longer or shorter.
    static final int wither_speed = 3;      //// Change to make the image wither faster or slower.
    static final int y_offset = 9;
    static final float gravity = 0.2f;

    public AlphabetTiles(String letter)
    {
        image.setFont(font);

        /**
         * Draw the outlined square.
         */
        image.setColor(Color.GRAY);
        image.fillRect(0, 0, square_size, square_size);
        image.setColor(Color.BLACK);
        image.drawRect(1,1, square_size-2, square_size-2);

        /**
         * Write on the letter.
         */
        image.setColor(Color.ORANGE);
        image.drawString(letter, square_size/2-2, square_size/2+2);

        /**
         * Show the image.
         */
        image.setTransparency(170);
        setImage(image);
    }

    /**
     * Act - do whatever the AlphabetTiles wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        try
        {
            if(has_been_on_screen < allotted_time)
            {
                if(!((myWorld) getWorld()).isGround(getX(), getY()+y_offset))
                {
                    ys += gravity;
                }
                else
                {
                    ys = 0;
                    y--;
                }

                y += ys;

                setLocation(getX(), y);

                has_been_on_screen++;
            }
            else
            {
                if(getImage().getTransparency() >= wither_speed)
                    getImage().setTransparency(getImage().getTransparency()-wither_speed);

                else
                {
                    getWorld().removeObject(this);
                    return;
                }
            }
        }
        catch(NullPointerException ex)
        {
            getWorld().removeObject(this);
            return;
        }
    }

    protected void addedToWorld(World world)
    {
        y = getY();
    }
}
