import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * The user controlled Elephant. Use your imagination!
 * 
 * I'm super paranoid about the addedToWorld(World world) error. So I check for that a lot.
 * 
 * Just change the commented things and you'll be all good.
 */
public class Elephant  extends Actor
{
    boolean can_run = true;
    boolean jump_is_on_ground = true;
    boolean just_hit = true;
    boolean added = false;
    boolean restarting = false;

    boolean can_scroll = false;

    int end_y_min = 306;
    int end_y_max = 346;
    int end_x_min = 643;
    int end_x_max = 670;

    GreenfootImage right[] = new GreenfootImage[3]; //// Increase this number if you make
    GreenfootImage left[] = new GreenfootImage[3];  //// more frames for your images.

    static final float anim_speed = 0.3f;  //// Change this for faster or slow animation.
    static final int angle = 6;            //// Change this for a more acute or obtuse angle.
    static final int max_jump = 20;        //// Change for a bigger or smaller jump.
    static final int x_offset = 12;         //// Change for a different player image size.
    static final int y_offset = 8;         //// Change if you use a different player image.
    static final float normal_gravity = 0.3f;  //// Should be the same as static final int gravity.
    static float gravity = 0.3f;     //// Change to alter the pull of gravity.
    static float jump_strength = 4;    //// Jump speed and height.
    static int speed = 4;                  //// Changes Elephant running speed.
    static final int bounce_strength = 12;  //// Amount to bounce off of bounce pads.

    private String last_letter = "";                   //// For alphabet recognition.

    int has_jumped = 0;
    boolean bouncing = false;

    double x, y;
    float xs, ys;

    boolean second;

    myWorld mworld;

    float frame = 0;
    boolean removed = false;

    public Elephant()
    {
        //Load the frames.

        if(right[0] == null)
        {
            right[0] = new GreenfootImage("Elephant.png");
            right[1] = new GreenfootImage("Elephant.png");
            right[2] = new GreenfootImage("Elephant.png");

            left[0] = new GreenfootImage("Elephant.png");
            left[1] = new GreenfootImage("Elephant.png");
            left[2] = new GreenfootImage("Elephant.png");

            for(int i = 0; i < 3; i++)
            {
                left[i].mirrorHorizontally();
                left[i].setTransparency(255);
                right[i].setTransparency(255);
            }
        }
    }

    public Elephant(boolean second)
    {
        this.second = second;

        //Load the frames.

        if(right[0] == null)
        {
            right[0] = new GreenfootImage("Elephant.png");
            right[1] = new GreenfootImage("Elephant.png");
            right[2] = new GreenfootImage("Elephant.png");

            left[0] = new GreenfootImage("Elephant.png");
            left[1] = new GreenfootImage("Elephant.png");
            left[2] = new GreenfootImage("Elephant.png");

            for(int i = 0; i < 3; i++)
            {
                left[i].mirrorHorizontally();
                left[i].setTransparency(255);
                right[i].setTransparency(255);
            }
        }
    }

    protected void addedToWorld(World world)
    {
        x = getX();
        y = getY();
        mworld = ((myWorld) getWorld());
        removed = false;
        added = true;

        mworld.scroll = (ScrollingFrame) getOneIntersectingObject(ScrollingFrame.class);
    }

    /**
     * Act - do whatever the Elephant wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act()
    {
        if(removed)
            mworld.can_act = false;

        if(!removed && mworld.can_act)
        {
            /**
             * Failsafe
             */
            if(removed)
                return;

            key_recognition();

            if(ys > 17)
                ys = 17;
            if(ys < -17)
                ys = -17;

            /**
             * Failsafe
             */
            if(removed)
                return;

            checkStop();

            /**
             * Different controls for different stages.
             */
            if(mworld.stage == 1)
            {
                end_y_min = 306;
                end_y_max = 350;
                end_x_min = 643;
                end_x_max = 670;
                normal_controls();
            }
            if(mworld.stage == 2)
                reversed_controls();
            if(mworld.stage == 3)
                normal_controls();
            if(mworld.stage == 4)
                mouse_controls();
            if(mworld.stage == 5)
                normal_controls();
            if(mworld.stage == 6)
            {
                gravity = normal_gravity;
                alphabet_detection();
                normal_controls();
            }
            if(mworld.stage == 7)
            {
                gravity = normal_gravity / 10;
                normal_controls();
            }
            if(mworld.stage == 8)
            {
                gravity = normal_gravity;
                no_jump_controls();
            }
            if(mworld.stage == 9)
                no_back_controls();
            if(mworld.stage == 10)
            {
                bouncing = false;
                wasd_controls();
            }
            if(mworld.stage == 11)
            {
                normal_controls();
                bouncing = true;
            }
            if(mworld.stage == 12)
            {
                bouncing = false;
                normal_controls();

                if(x >= 236 && y >= 362 && !mworld.show_path)
                    mworld.show_path = true;
                else if((x < 256 || y < 362) || (x < 278 && y < 362))
                    mworld.show_path = false;
            }
            if(mworld.stage == 13)
                no_jump_controls();
            if(mworld.stage == 14)
            {
                if(Greenfoot.isKeyDown("enter") && !mworld.button_has_been_tripped)
                    mworld.door_can_open = true;

                normal_controls();
            }
            if(mworld.stage == 15)
            {
                end_x_min = 583;
                end_x_max = 600;
                end_y_min = 139;
                end_y_max = 143;
                normal_controls();
            }
            if(mworld.stage == 16)
                normal_controls();
            if(mworld.stage == 17)
            {
                /**
                 * Just in case the player comes off of the ScrollingFrame. It's possible.
                 */
                try
                {
                    scroll_controls();
                }
                catch(NullPointerException ex)
                {
                    Greenfoot.stop();
                }

                if(y < 0 && !can_scroll)
                    can_scroll = true;

                if(y < 0 && xs < 0)
                    xs = 0;

                if(can_scroll)
                {
                    if((y < 0 && mworld.scroll_position > -697) || (y > 0 && mworld.scroll_position > -697))
                    {
                        mworld.scroll_speed = (int)-xs;
                        xs = 0;
                    }
                    else
                        mworld.scroll_speed = 0;
                }
            }
            if(mworld.stage == 18)
                normal_controls();
            if(mworld.stage == 19)
                normal_controls();
            if(mworld.stage == 20)
            {
                normal_controls();
                jump_strength = 4.2f;
            }
            if(mworld.stage == 21)
            {
                jump_strength = 4;
                normal_controls();
                getImage().setTransparency(0);
            }
            if(mworld.stage == 22)
            {
                if(!second)
                    normal_controls();
                else if(second)
                    reversed_controls();
            }

            /**
             * Failsafe
             */
            if(!removed && mworld.can_act && mworld.stage != 17)
            {
                checkBackground();
            }

            else if(!removed && mworld.can_act && mworld.stage == 17)
            {
                checkScrollBackground();
            }

            /**
             * Fall through a pit in the bottom.
             */
            if(y > mworld.height + 30)
                y = -30;

            /**
             * Incase the player falls through the ground as it so often does.
             */
            if(mworld.stage != 12 && y > mworld.height+20)
            {
                x = mworld.spawn_x;
                y = mworld.spawn_y;
            }

            if(mworld.stage == 12 && y > mworld.height+20 && x < 426)
            {
                x = mworld.spawn_x;
                y = mworld.spawn_y;
            }

            /**
             * Failsafe
             */
            if(removed)     /////// Need to end the program when the elephant gets removed.
                return;     /////// Otherwise you'll error out.

            move();
        }
    }   

    private void key_recognition()
    {
        if(Greenfoot.isKeyDown("F1") && !restarting)
        {
            restarting = true;
        }
        if(!Greenfoot.isKeyDown("F1") && restarting)
        {
            ((myWorld) getWorld()).restartStage();
            removed = true;
        }
    }

    private void setLocation(double X, double Y)
    {
        setLocation((int) X, (int) Y);
    }

    public void move()
    {
        x += xs;
        y += ys;
        setLocation(x, y);
    }

    public void checkStop()
    {
        if(!Greenfoot.isKeyDown("right") && !Greenfoot.isKeyDown("left"))
        {
            if(xs > 0)
            {
                frame = 0;
                setImage(right[(int)frame]);
            }
            else if(xs < 0)
            {
                frame = 0;
                setImage(left[(int)frame]);
            }
            xs = 0;
            setRotation(0);
        }
    }

    public void checkBackground()
    {
        /**
         * Failsafe
         */
        if(removed)     /////// Need to end the program when the elephant gets removed.
            return;     /////// Otherwise you'll error out.

        if(mworld.isHurt(getX()-x_offset, getY()) ||
        mworld.isHurt(getX()+x_offset, getY()) ||
        mworld.isHurt(getX(), getY()+y_offset) ||
        mworld.isHurt(getX(), getY()-y_offset))
        {
            getWorld().addObject(new Carcass(), getX(), getY());
            mworld.deaths++;
            removed = true;
            mworld.restartStage();
            return;
        }

        /**
         * Failsafe
         */
        if(removed)     /////// Need to end the program when the elephant gets removed.
            return;     /////// Otherwise you'll error out.

        /**
         * gravity, and ground at feet.
         */

        if(mworld.isGround(x, y))
        {
            y -= getImage().getWidth()/2;
        }

        if(has_jumped >= max_jump && !mworld.isGround(getX(), getY()+y_offset))
        {
            ys += gravity;
        }
        else if(mworld.isGround(getX(), getY()+y_offset) && bouncing)
        {
            has_jumped = 0;
            ys = -ys*0.98f;
        }

        else if(mworld.isGround(getX(), getY()+y_offset) && !bouncing)
        {
            y -= 0.5;
            has_jumped = 0;
            ys = 0;
        }

        else if(!mworld.isGround(getX(), getY()+y_offset) && has_jumped < max_jump)
        {
            ys += gravity;
        }

        /**
         * Ground, walls, and bounce in other places.
         */

        if(mworld.isBounce(getX(), getY()+y_offset))
        {
            ys = -bounce_strength;
        }

        if(mworld.isGround(getX()+x_offset, getY()))
        {
            xs = 0;
            x--;
        }
        if(mworld.isGround(getX(), getY()+y_offset+2) && just_hit)
        {
            just_hit = false;

            y--;
        }
        else if(!mworld.isGround(getX(), getY()+y_offset+2))
        {
            just_hit = true;
        }

        if(mworld.isGround(getX()-x_offset, getY()))
        {
            xs = 0;
            x++;
        }

        if(mworld.isGround(getX(), getY()+y_offset+3) && !mworld.isGround(getX(), getY()+y_offset))
        {
            y -= 0.5;
        }

        if(mworld.isGround(getX(), getY()-y_offset) && !bouncing)
        {
            ys = 0;
            y++;
        }

        if(mworld.isGround(getX(), getY()-y_offset) && bouncing)
        {
            ys = -ys;
            y++;
            setLocation((int) x, (int) y);
        }

        if(x >= end_x_min && x <= end_x_max && y >= end_y_min && y <= end_y_max)
        {
            mworld.nextStage();
            getWorld().removeObject(this);
            removed = true;
            return;
        }

        if(x >= end_x_min && x <= end_x_max && y >= end_y_min && y <= end_y_max)
        {
            mworld.nextStage();
            getWorld().removeObject(this);
            removed = true;
            return;
        }

        if(removed)     /////// Need to end the program when the elephant gets removed.
            getWorld().removeObject(this);     /////// Otherwise you'll error out.
    }

    ///_____________________///

    public void checkScrollBackground()
    {
        /**
         * Failsafe
         */
        if(removed)     /////// Need to end the program when the elephant gets removed.
            return;     /////// Otherwise you'll error out.

        if(mworld.scroll.isHurt(getX()-x_offset, getY()) ||
        mworld.scroll.isHurt(getX()+x_offset, getY()) ||
        mworld.scroll.isHurt(getX(), getY()+y_offset) ||
        mworld.scroll.isHurt(getX(), getY()-y_offset))
        {
            getWorld().addObject(new Carcass(), getX(), getY());
            mworld.deaths++;
            removed = true;
            mworld.restartStage();
            return;
        }

        /**
         * Failsafe
         */
        if(removed)     /////// Need to end the program when the elephant gets removed.
            return;     /////// Otherwise you'll error out.

        /**
         * gravity, and ground at feet.
         */

        if(mworld.scroll.isGround(x, y))
        {
            y -= getImage().getWidth()/2;
        }

        if(has_jumped >= max_jump && !mworld.scroll.isGround(getX(), getY()+y_offset))
        {
            ys += gravity;
        }
        else if(mworld.scroll.isGround(getX(), getY()+y_offset) && bouncing)
        {
            ys = -(ys/2);
            y -= 0.5;
            has_jumped = 0;
        }

        else if(mworld.scroll.isGround(getX(), getY()+y_offset) && !bouncing)
        {
            ys = -ys/2;

            y -= 0.5;
            has_jumped = 0;
            ys = 0;
        }

        if(mworld.scroll.isGround(x, y+y_offset/2+3))
        {
            y--;
        }

        else if(!mworld.scroll.isGround(getX(), getY()+y_offset) && has_jumped < max_jump)
        {
            ys += gravity;
        }

        /**
         * Ground, walls, and bounce in other places.
         */

        if(mworld.scroll.isBounce(getX(), getY()+y_offset))
        {
            ys = -bounce_strength;
        }

        if(mworld.scroll.isGround(getX()+x_offset, getY()))
        {
            xs = 0;
            x--;
        }
        if(mworld.scroll.isGround(getX(), getY()+y_offset+2) && just_hit)
        {
            just_hit = false;

            y--;
        }
        else if(!mworld.scroll.isGround(getX(), getY()+y_offset+2))
        {
            just_hit = true;
        }

        if(mworld.scroll.isGround(getX()-x_offset, getY()))
        {
            xs = 0;
            x++;
        }

        if(mworld.scroll.isGround(getX(), getY()+y_offset+3) && !mworld.scroll.isGround(getX(), getY()+y_offset))
        {
            y -= 0.5;
        }

        if(mworld.scroll.isGround(getX(), getY()-y_offset) && !bouncing)
        {
            ys = 0;
            y++;
        }

        if(mworld.scroll.isGround(getX(), getY()-y_offset) && bouncing)
        {
            ys = -ys;
            y++;
            setLocation((int) x, (int) y);
        }

        if(mworld.scroll.isWarp(getX(), getY()+5) && Greenfoot.isKeyDown("down"))
        {
            mworld.nextStage();
            getWorld().removeObject(this);
            removed = true;
            return;
        }

        if(removed)     /////// Need to end the program when the elephant gets removed.
            getWorld().removeObject(this);     /////// Otherwise you'll error out.
    }

    ///_____________________///

    public void shake(int direction)
    {
        if(direction == 0)
        {
            setImage(right[(int)frame]);
        }

        else if(direction == 16)
        {
            setImage(left[(int)frame]);
        }

        /////SHAKE ME/////
        if((int) frame == 1)
            setRotation(-angle);

        if((int) frame == 2)
            setRotation(angle);

        ////SET FRAME/////
        if(frame + anim_speed < 3)
            frame += anim_speed;
        else
            frame = 0;
    }

    /**
     * All the movement determining happens here. Every stage is different.
     */
    public void normal_controls()
    {
        normal_jump();

        if(Greenfoot.isKeyDown("right"))
        {
            shake(0);
            xs = speed;
        }

        if(Greenfoot.isKeyDown("left"))
        {
            shake(16);
            xs = -speed;
        }
    }

    public void scroll_controls()
    {
        if(y > 0)
            scroll_jump();

        if(Greenfoot.isKeyDown("right"))
        {
            shake(0);
            xs = speed;
        }

        if(Greenfoot.isKeyDown("left"))
        {
            shake(16);
            xs = -speed;
        }
    }

    public void normal_jump()
    {
        if(Greenfoot.isKeyDown("up") && has_jumped < max_jump && jump_is_on_ground)
        {
            ys = -jump_strength;
            has_jumped++;
        }
        if(!Greenfoot.isKeyDown("up") && !mworld.isGround(x,y+y_offset+1))
        {
            jump_is_on_ground = false;
        }
        if(((myWorld) getWorld()).isGround(x,y+y_offset) && !jump_is_on_ground)
        {
            jump_is_on_ground = true;
        }
    }

    public void scroll_jump()
    {
        if(Greenfoot.isKeyDown("up") && has_jumped < max_jump && jump_is_on_ground)
        {
            ys = -jump_strength;
            has_jumped++;
        }
        if(!Greenfoot.isKeyDown("up") && !mworld.scroll.isGround(x,y+y_offset+1))
        {
            jump_is_on_ground = false;
        }
        if(mworld.scroll.isGround(x,y+y_offset) && !jump_is_on_ground)
        {
            jump_is_on_ground = true;
        }
    }

    public void wasd_controls()
    {
        wasd_jump();

        if(Greenfoot.isKeyDown("d"))
        {
            shake(0);
            xs = speed;
        }

        if(Greenfoot.isKeyDown("a"))
        {
            shake(16);
            xs = -speed;
        }
    }

    public void wasd_jump()
    {
        if(Greenfoot.isKeyDown("w") && has_jumped < max_jump && jump_is_on_ground)
        {
            ys = -jump_strength;
            has_jumped++;
        }
        if(!Greenfoot.isKeyDown("w") && !mworld.isGround(x,y+y_offset))
        {
            jump_is_on_ground = false;
        }
        if(((myWorld) getWorld()).isGround(x,y+y_offset) && !jump_is_on_ground)
        {
            jump_is_on_ground = true;
        }
    }

    public void no_jump_controls()
    {
        if(Greenfoot.isKeyDown("right"))
        {
            shake(0);
            xs = speed;
        }

        if(Greenfoot.isKeyDown("left"))
        {
            shake(16);
            xs = -speed;
        }
    }

    public void no_back_controls()
    {
        normal_jump();

        if(Greenfoot.isKeyDown("right"))
        {
            shake(0);
            xs = speed;
        }

        if(Greenfoot.isKeyDown("left"))
        {
            shake(16);
        }
    }

    public void reversed_controls()
    {
        normal_jump();

        if(Greenfoot.isKeyDown("left"))
        {
            shake(0);
            xs = speed;
        }

        if(Greenfoot.isKeyDown("right"))
        {
            shake(16);
            xs = -speed;
        }
    }

    public void mouse_controls()
    {
        MouseInfo mouse = Greenfoot.getMouseInfo();

        if(Greenfoot.isKeyDown("right"))
        {
            shake(0);
        }

        if(Greenfoot.isKeyDown("left"))
        {
            shake(16);
        }

        if(Greenfoot.mouseClicked(null))
        {
            x = mouse.getX();
            y = mouse.getY();
        }
    }

    public void alphabet_detection()
    {
        /**
         * Yeah, this is a crappy way to do it. But it's done and it works well, so oh well.
         */
        if(Greenfoot.isKeyDown("a") && last_letter == "")
        {
            getWorld().addObject(new AlphabetTiles("a"), (int) x+13, (int) y-20);
            last_letter = "a";
        }
        if(Greenfoot.isKeyDown("b") && last_letter == "a")
        {
            getWorld().addObject(new AlphabetTiles("b"), (int) x+13, (int) y-20);
            last_letter = "b";
        }
        if(Greenfoot.isKeyDown("c") && last_letter == "b")
        {
            getWorld().addObject(new AlphabetTiles("c"), (int) x+13, (int) y-20);
            last_letter = "c";
        }
        if(Greenfoot.isKeyDown("d") && last_letter == "c")
        {
            getWorld().addObject(new AlphabetTiles("d"), (int) x+13, (int) y-20);
            last_letter = "d";
        }
        if(Greenfoot.isKeyDown("e") && last_letter == "d")
        {
            getWorld().addObject(new AlphabetTiles("b"), (int) x+13, (int) y-20);
            last_letter = "e";
        }
        if(Greenfoot.isKeyDown("f") && last_letter == "e")
        {
            getWorld().addObject(new AlphabetTiles("f"), (int) x+13, (int) y-20);
            last_letter = "f";
        }
        if(Greenfoot.isKeyDown("g") && last_letter == "f")
        {
            getWorld().addObject(new AlphabetTiles("g"), (int) x+13, (int) y-20);
            last_letter = "g";
        }
        if(Greenfoot.isKeyDown("h") && last_letter == "g")
        {
            getWorld().addObject(new AlphabetTiles("h"), (int) x+13, (int) y-20);
            last_letter = "h";
        }
        if(Greenfoot.isKeyDown("i") && last_letter == "h")
        {
            getWorld().addObject(new AlphabetTiles("i"), (int) x+13, (int) y-20);
            last_letter = "i";
        }
        if(Greenfoot.isKeyDown("j") && last_letter == "i")
        {
            getWorld().addObject(new AlphabetTiles("j"), (int) x+13, (int) y-20);
            last_letter = "j";
        }
        if(Greenfoot.isKeyDown("k") && last_letter == "j")
        {
            getWorld().addObject(new AlphabetTiles("k"), (int) x+13, (int) y-20);
            last_letter = "k";
        }
        if(Greenfoot.isKeyDown("l") && last_letter == "k")
        {
            getWorld().addObject(new AlphabetTiles("l"), (int) x+13, (int) y-20);
            last_letter = "l";
        }
        if(Greenfoot.isKeyDown("m") && last_letter == "l")
        {
            getWorld().addObject(new AlphabetTiles("m"), (int) x+13, (int) y-20);
            last_letter = "m";
        }
        if(Greenfoot.isKeyDown("n") && last_letter == "m")
        {
            getWorld().addObject(new AlphabetTiles("n"), (int) x+13, (int) y-20);
            last_letter = "n";
        }
        if(Greenfoot.isKeyDown("o") && last_letter == "n")
        {
            getWorld().addObject(new AlphabetTiles("o"), (int) x+13, (int) y-20);
            last_letter = "o";
        }
        if(Greenfoot.isKeyDown("p") && last_letter == "o")
        {
            getWorld().addObject(new AlphabetTiles("p"), (int) x+13, (int) y-20);
            last_letter = "p";
        }
        if(Greenfoot.isKeyDown("q") && last_letter == "p")
        {
            getWorld().addObject(new AlphabetTiles("q"), (int) x+13, (int) y-20);
            last_letter = "q";
        }
        if(Greenfoot.isKeyDown("r") && last_letter == "q")
        {
            getWorld().addObject(new AlphabetTiles("r"), (int) x+13, (int) y-20);
            last_letter = "r";
        }
        if(Greenfoot.isKeyDown("s") && last_letter == "r")
        {
            getWorld().addObject(new AlphabetTiles("s"), (int) x+13, (int) y-20);
            last_letter = "s";
        }
        if(Greenfoot.isKeyDown("t") && last_letter == "s")
        {
            getWorld().addObject(new AlphabetTiles("t"), (int) x+13, (int) y-20);
            last_letter = "t";
        }
        if(Greenfoot.isKeyDown("u") && last_letter == "t")
        {
            getWorld().addObject(new AlphabetTiles("u"), (int) x+13, (int) y-20);
            last_letter = "u";
        }
        if(Greenfoot.isKeyDown("v") && last_letter == "u")
        {
            getWorld().addObject(new AlphabetTiles("v"), (int) x+13, (int) y-20);
            last_letter = "v";
        }
        if(Greenfoot.isKeyDown("w") && last_letter == "v")
        {
            getWorld().addObject(new AlphabetTiles("w"), (int) x+13, (int) y-20);
            last_letter = "w";
        }
        if(Greenfoot.isKeyDown("x") && last_letter == "w")
        {
            getWorld().addObject(new AlphabetTiles("x"), (int) x+13, (int) y-20);
            last_letter = "x";
        }
        if(Greenfoot.isKeyDown("y") && last_letter == "x")
        {
            getWorld().addObject(new AlphabetTiles("y"), (int) x+13, (int) y-20);
            last_letter = "y";
        }
        if(Greenfoot.isKeyDown("z") && last_letter == "y")
        {
            getWorld().addObject(new AlphabetTiles("z"), (int) x+13, (int) y-20);
            mworld.door_can_open = true;
            last_letter = "z";
        }
    }
}
